from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.creation_origin import CreationOrigin
from ..models.fields import Fields
from ..models.workflow_output_summary import WorkflowOutputSummary
from ..models.workflow_task_group_summary import WorkflowTaskGroupSummary
from ..models.workflow_task_summary import WorkflowTaskSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowOutput")


@attr.s(auto_attribs=True, repr=False)
class WorkflowOutput:
    """  """

    _next_outputs: Union[Unset, List[WorkflowOutputSummary]] = UNSET
    _next_tasks: Union[Unset, List[WorkflowTaskSummary]] = UNSET
    _source_outputs: Union[Unset, List[WorkflowOutputSummary]] = UNSET
    _source_tasks: Union[Unset, List[WorkflowTaskSummary]] = UNSET
    _creation_origin: Union[Unset, CreationOrigin] = UNSET
    _fields: Union[Unset, Fields] = UNSET
    _task: Union[Unset, WorkflowTaskSummary] = UNSET
    _web_url: Union[Unset, str] = UNSET
    _workflow_task_group: Union[Unset, WorkflowTaskGroupSummary] = UNSET
    _display_id: Union[Unset, str] = UNSET
    _id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("next_outputs={}".format(repr(self._next_outputs)))
        fields.append("next_tasks={}".format(repr(self._next_tasks)))
        fields.append("source_outputs={}".format(repr(self._source_outputs)))
        fields.append("source_tasks={}".format(repr(self._source_tasks)))
        fields.append("creation_origin={}".format(repr(self._creation_origin)))
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("task={}".format(repr(self._task)))
        fields.append("web_url={}".format(repr(self._web_url)))
        fields.append("workflow_task_group={}".format(repr(self._workflow_task_group)))
        fields.append("display_id={}".format(repr(self._display_id)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WorkflowOutput({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        next_outputs: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._next_outputs, Unset):
            next_outputs = []
            for next_outputs_item_data in self._next_outputs:
                next_outputs_item = next_outputs_item_data.to_dict()

                next_outputs.append(next_outputs_item)

        next_tasks: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._next_tasks, Unset):
            next_tasks = []
            for next_tasks_item_data in self._next_tasks:
                next_tasks_item = next_tasks_item_data.to_dict()

                next_tasks.append(next_tasks_item)

        source_outputs: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._source_outputs, Unset):
            source_outputs = []
            for source_outputs_item_data in self._source_outputs:
                source_outputs_item = source_outputs_item_data.to_dict()

                source_outputs.append(source_outputs_item)

        source_tasks: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._source_tasks, Unset):
            source_tasks = []
            for source_tasks_item_data in self._source_tasks:
                source_tasks_item = source_tasks_item_data.to_dict()

                source_tasks.append(source_tasks_item)

        creation_origin: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._creation_origin, Unset):
            creation_origin = self._creation_origin.to_dict()

        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._fields, Unset):
            fields = self._fields.to_dict()

        task: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._task, Unset):
            task = self._task.to_dict()

        web_url = self._web_url
        workflow_task_group: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._workflow_task_group, Unset):
            workflow_task_group = self._workflow_task_group.to_dict()

        display_id = self._display_id
        id = self._id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if next_outputs is not UNSET:
            field_dict["nextOutputs"] = next_outputs
        if next_tasks is not UNSET:
            field_dict["nextTasks"] = next_tasks
        if source_outputs is not UNSET:
            field_dict["sourceOutputs"] = source_outputs
        if source_tasks is not UNSET:
            field_dict["sourceTasks"] = source_tasks
        if creation_origin is not UNSET:
            field_dict["creationOrigin"] = creation_origin
        if fields is not UNSET:
            field_dict["fields"] = fields
        if task is not UNSET:
            field_dict["task"] = task
        if web_url is not UNSET:
            field_dict["webURL"] = web_url
        if workflow_task_group is not UNSET:
            field_dict["workflowTaskGroup"] = workflow_task_group
        if display_id is not UNSET:
            field_dict["displayId"] = display_id
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_next_outputs() -> Union[Unset, List[WorkflowOutputSummary]]:
            next_outputs = []
            _next_outputs = d.pop("nextOutputs")
            for next_outputs_item_data in _next_outputs or []:
                next_outputs_item = WorkflowOutputSummary.from_dict(next_outputs_item_data, strict=False)

                next_outputs.append(next_outputs_item)

            return next_outputs

        try:
            next_outputs = get_next_outputs()
        except KeyError:
            if strict:
                raise
            next_outputs = cast(Union[Unset, List[WorkflowOutputSummary]], UNSET)

        def get_next_tasks() -> Union[Unset, List[WorkflowTaskSummary]]:
            next_tasks = []
            _next_tasks = d.pop("nextTasks")
            for next_tasks_item_data in _next_tasks or []:
                next_tasks_item = WorkflowTaskSummary.from_dict(next_tasks_item_data, strict=False)

                next_tasks.append(next_tasks_item)

            return next_tasks

        try:
            next_tasks = get_next_tasks()
        except KeyError:
            if strict:
                raise
            next_tasks = cast(Union[Unset, List[WorkflowTaskSummary]], UNSET)

        def get_source_outputs() -> Union[Unset, List[WorkflowOutputSummary]]:
            source_outputs = []
            _source_outputs = d.pop("sourceOutputs")
            for source_outputs_item_data in _source_outputs or []:
                source_outputs_item = WorkflowOutputSummary.from_dict(source_outputs_item_data, strict=False)

                source_outputs.append(source_outputs_item)

            return source_outputs

        try:
            source_outputs = get_source_outputs()
        except KeyError:
            if strict:
                raise
            source_outputs = cast(Union[Unset, List[WorkflowOutputSummary]], UNSET)

        def get_source_tasks() -> Union[Unset, List[WorkflowTaskSummary]]:
            source_tasks = []
            _source_tasks = d.pop("sourceTasks")
            for source_tasks_item_data in _source_tasks or []:
                source_tasks_item = WorkflowTaskSummary.from_dict(source_tasks_item_data, strict=False)

                source_tasks.append(source_tasks_item)

            return source_tasks

        try:
            source_tasks = get_source_tasks()
        except KeyError:
            if strict:
                raise
            source_tasks = cast(Union[Unset, List[WorkflowTaskSummary]], UNSET)

        def get_creation_origin() -> Union[Unset, CreationOrigin]:
            creation_origin: Union[Unset, Union[Unset, CreationOrigin]] = UNSET
            _creation_origin = d.pop("creationOrigin")

            if not isinstance(_creation_origin, Unset):
                creation_origin = CreationOrigin.from_dict(_creation_origin)

            return creation_origin

        try:
            creation_origin = get_creation_origin()
        except KeyError:
            if strict:
                raise
            creation_origin = cast(Union[Unset, CreationOrigin], UNSET)

        def get_fields() -> Union[Unset, Fields]:
            fields: Union[Unset, Union[Unset, Fields]] = UNSET
            _fields = d.pop("fields")

            if not isinstance(_fields, Unset):
                fields = Fields.from_dict(_fields)

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Unset, Fields], UNSET)

        def get_task() -> Union[Unset, WorkflowTaskSummary]:
            task: Union[Unset, Union[Unset, WorkflowTaskSummary]] = UNSET
            _task = d.pop("task")

            if not isinstance(_task, Unset):
                task = WorkflowTaskSummary.from_dict(_task)

            return task

        try:
            task = get_task()
        except KeyError:
            if strict:
                raise
            task = cast(Union[Unset, WorkflowTaskSummary], UNSET)

        def get_web_url() -> Union[Unset, str]:
            web_url = d.pop("webURL")
            return web_url

        try:
            web_url = get_web_url()
        except KeyError:
            if strict:
                raise
            web_url = cast(Union[Unset, str], UNSET)

        def get_workflow_task_group() -> Union[Unset, WorkflowTaskGroupSummary]:
            workflow_task_group: Union[Unset, Union[Unset, WorkflowTaskGroupSummary]] = UNSET
            _workflow_task_group = d.pop("workflowTaskGroup")

            if not isinstance(_workflow_task_group, Unset):
                workflow_task_group = WorkflowTaskGroupSummary.from_dict(_workflow_task_group)

            return workflow_task_group

        try:
            workflow_task_group = get_workflow_task_group()
        except KeyError:
            if strict:
                raise
            workflow_task_group = cast(Union[Unset, WorkflowTaskGroupSummary], UNSET)

        def get_display_id() -> Union[Unset, str]:
            display_id = d.pop("displayId")
            return display_id

        try:
            display_id = get_display_id()
        except KeyError:
            if strict:
                raise
            display_id = cast(Union[Unset, str], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        workflow_output = cls(
            next_outputs=next_outputs,
            next_tasks=next_tasks,
            source_outputs=source_outputs,
            source_tasks=source_tasks,
            creation_origin=creation_origin,
            fields=fields,
            task=task,
            web_url=web_url,
            workflow_task_group=workflow_task_group,
            display_id=display_id,
            id=id,
        )

        workflow_output.additional_properties = d
        return workflow_output

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def next_outputs(self) -> List[WorkflowOutputSummary]:
        if isinstance(self._next_outputs, Unset):
            raise NotPresentError(self, "next_outputs")
        return self._next_outputs

    @next_outputs.setter
    def next_outputs(self, value: List[WorkflowOutputSummary]) -> None:
        self._next_outputs = value

    @next_outputs.deleter
    def next_outputs(self) -> None:
        self._next_outputs = UNSET

    @property
    def next_tasks(self) -> List[WorkflowTaskSummary]:
        if isinstance(self._next_tasks, Unset):
            raise NotPresentError(self, "next_tasks")
        return self._next_tasks

    @next_tasks.setter
    def next_tasks(self, value: List[WorkflowTaskSummary]) -> None:
        self._next_tasks = value

    @next_tasks.deleter
    def next_tasks(self) -> None:
        self._next_tasks = UNSET

    @property
    def source_outputs(self) -> List[WorkflowOutputSummary]:
        if isinstance(self._source_outputs, Unset):
            raise NotPresentError(self, "source_outputs")
        return self._source_outputs

    @source_outputs.setter
    def source_outputs(self, value: List[WorkflowOutputSummary]) -> None:
        self._source_outputs = value

    @source_outputs.deleter
    def source_outputs(self) -> None:
        self._source_outputs = UNSET

    @property
    def source_tasks(self) -> List[WorkflowTaskSummary]:
        if isinstance(self._source_tasks, Unset):
            raise NotPresentError(self, "source_tasks")
        return self._source_tasks

    @source_tasks.setter
    def source_tasks(self, value: List[WorkflowTaskSummary]) -> None:
        self._source_tasks = value

    @source_tasks.deleter
    def source_tasks(self) -> None:
        self._source_tasks = UNSET

    @property
    def creation_origin(self) -> CreationOrigin:
        if isinstance(self._creation_origin, Unset):
            raise NotPresentError(self, "creation_origin")
        return self._creation_origin

    @creation_origin.setter
    def creation_origin(self, value: CreationOrigin) -> None:
        self._creation_origin = value

    @creation_origin.deleter
    def creation_origin(self) -> None:
        self._creation_origin = UNSET

    @property
    def fields(self) -> Fields:
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Fields) -> None:
        self._fields = value

    @fields.deleter
    def fields(self) -> None:
        self._fields = UNSET

    @property
    def task(self) -> WorkflowTaskSummary:
        if isinstance(self._task, Unset):
            raise NotPresentError(self, "task")
        return self._task

    @task.setter
    def task(self, value: WorkflowTaskSummary) -> None:
        self._task = value

    @task.deleter
    def task(self) -> None:
        self._task = UNSET

    @property
    def web_url(self) -> str:
        """ URL of the workflow output """
        if isinstance(self._web_url, Unset):
            raise NotPresentError(self, "web_url")
        return self._web_url

    @web_url.setter
    def web_url(self, value: str) -> None:
        self._web_url = value

    @web_url.deleter
    def web_url(self) -> None:
        self._web_url = UNSET

    @property
    def workflow_task_group(self) -> WorkflowTaskGroupSummary:
        if isinstance(self._workflow_task_group, Unset):
            raise NotPresentError(self, "workflow_task_group")
        return self._workflow_task_group

    @workflow_task_group.setter
    def workflow_task_group(self, value: WorkflowTaskGroupSummary) -> None:
        self._workflow_task_group = value

    @workflow_task_group.deleter
    def workflow_task_group(self) -> None:
        self._workflow_task_group = UNSET

    @property
    def display_id(self) -> str:
        """ User-friendly ID of the workflow task group """
        if isinstance(self._display_id, Unset):
            raise NotPresentError(self, "display_id")
        return self._display_id

    @display_id.setter
    def display_id(self, value: str) -> None:
        self._display_id = value

    @display_id.deleter
    def display_id(self) -> None:
        self._display_id = UNSET

    @property
    def id(self) -> str:
        """ The ID of the workflow output """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.session_message_style import SessionMessageStyle
from ..types import UNSET, Unset

T = TypeVar("T", bound="SessionMessageCreate")


@attr.s(auto_attribs=True, repr=False)
class SessionMessageCreate:
    """  """

    _content: str
    _style: Union[Unset, SessionMessageStyle] = SessionMessageStyle.NONE

    def __repr__(self):
        fields = []
        fields.append("content={}".format(repr(self._content)))
        fields.append("style={}".format(repr(self._style)))
        return "SessionMessageCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        content = self._content
        style: Union[Unset, int] = UNSET
        if not isinstance(self._style, Unset):
            style = self._style.value

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if content is not UNSET:
            field_dict["content"] = content
        if style is not UNSET:
            field_dict["style"] = style

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_content() -> str:
            content = d.pop("content")
            return content

        try:
            content = get_content()
        except KeyError:
            if strict:
                raise
            content = cast(str, UNSET)

        def get_style() -> Union[Unset, SessionMessageStyle]:
            style = UNSET
            _style = d.pop("style")
            if _style is not None and _style is not UNSET:
                try:
                    style = SessionMessageStyle(_style)
                except ValueError:
                    style = SessionMessageStyle.of_unknown(_style)

            return style

        try:
            style = get_style()
        except KeyError:
            if strict:
                raise
            style = cast(Union[Unset, SessionMessageStyle], UNSET)

        session_message_create = cls(
            content=content,
            style=style,
        )

        return session_message_create

    @property
    def content(self) -> str:
        """ A message string, to be rendered as plain text with Benchling chips. References to Benchling items (up to 10 per msg) will be rendered as chips in the Benchling UX. A valid reference is a Benchling API id, prefixed with "id:" and contained by braces. For example: "{id:ent_a0SApq3}." """
        if isinstance(self._content, Unset):
            raise NotPresentError(self, "content")
        return self._content

    @content.setter
    def content(self, value: str) -> None:
        self._content = value

    @property
    def style(self) -> SessionMessageStyle:
        if isinstance(self._style, Unset):
            raise NotPresentError(self, "style")
        return self._style

    @style.setter
    def style(self, value: SessionMessageStyle) -> None:
        self._style = value

    @style.deleter
    def style(self) -> None:
        self._style = UNSET



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.dataset import Dataset
from ...models.forbidden_error import ForbiddenError
from ...models.get_dataset_content_format import GetDatasetContentFormat
from ...models.not_found_error import NotFoundError
from ...types import Response, UNSET, Unset


def _get_kwargs(
    *,
    client: Client,
    dataset_id: str,
    content_format: Union[Unset, GetDatasetContentFormat] = UNSET,
) -> Dict[str, Any]:
    url = "{}/datasets/{dataset_id}".format(client.base_url, dataset_id=dataset_id)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    json_content_format: Union[Unset, int] = UNSET
    if not isinstance(content_format, Unset):
        json_content_format = content_format.value

    params: Dict[str, Any] = {}
    if not isinstance(json_content_format, Unset) and json_content_format is not None:
        params["contentFormat"] = json_content_format

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Dataset, ForbiddenError, NotFoundError]]:
    if response.status_code == 200:
        response_200 = Dataset.from_dict(response.json(), strict=False)

        return response_200
    if response.status_code == 403:
        response_403 = ForbiddenError.from_dict(response.json(), strict=False)

        return response_403
    if response.status_code == 404:
        response_404 = NotFoundError.from_dict(response.json(), strict=False)

        return response_404
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Dataset, ForbiddenError, NotFoundError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    dataset_id: str,
    content_format: Union[Unset, GetDatasetContentFormat] = UNSET,
) -> Response[Union[Dataset, ForbiddenError, NotFoundError]]:
    kwargs = _get_kwargs(
        client=client,
        dataset_id=dataset_id,
        content_format=content_format,
    )

    response = client.httpx_client.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    dataset_id: str,
    content_format: Union[Unset, GetDatasetContentFormat] = UNSET,
) -> Optional[Union[Dataset, ForbiddenError, NotFoundError]]:
    """Get a dataset, which is a tabular data construct that stores typed columns and rows of data. Call this endpoint to download the data.
    If the Dataset has `SUCCEEDED` status, the response will contain a `Content-Location` header with a URL to download the Dataset in the requested `contentFormat`. If the Dataset has `FAILED_VALIDATION` or `IN_PROGRESS ` status, the `Content-Location` header will always be empty.
    """

    return sync_detailed(
        client=client,
        dataset_id=dataset_id,
        content_format=content_format,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    dataset_id: str,
    content_format: Union[Unset, GetDatasetContentFormat] = UNSET,
) -> Response[Union[Dataset, ForbiddenError, NotFoundError]]:
    kwargs = _get_kwargs(
        client=client,
        dataset_id=dataset_id,
        content_format=content_format,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    dataset_id: str,
    content_format: Union[Unset, GetDatasetContentFormat] = UNSET,
) -> Optional[Union[Dataset, ForbiddenError, NotFoundError]]:
    """Get a dataset, which is a tabular data construct that stores typed columns and rows of data. Call this endpoint to download the data.
    If the Dataset has `SUCCEEDED` status, the response will contain a `Content-Location` header with a URL to download the Dataset in the requested `contentFormat`. If the Dataset has `FAILED_VALIDATION` or `IN_PROGRESS ` status, the `Content-Location` header will always be empty.
    """

    return (
        await asyncio_detailed(
            client=client,
            dataset_id=dataset_id,
            content_format=content_format,
        )
    ).parsed



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class NamingStrategy(Enums.KnownString):
    NEW_IDS = "NEW_IDS"
    IDS_FROM_NAMES = "IDS_FROM_NAMES"
    DELETE_NAMES = "DELETE_NAMES"
    SET_FROM_NAME_PARTS = "SET_FROM_NAME_PARTS"
    REPLACE_NAMES_FROM_PARTS = "REPLACE_NAMES_FROM_PARTS"
    KEEP_NAMES = "KEEP_NAMES"
    REPLACE_ID_AND_NAME_FROM_PARTS = "REPLACE_ID_AND_NAME_FROM_PARTS"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "NamingStrategy":
        if not isinstance(val, str):
            raise ValueError(f"Value of NamingStrategy must be a string (encountered: {val})")
        newcls = Enum("NamingStrategy", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(NamingStrategy, getattr(newcls, "_UNKNOWN"))



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.workflow_flowchart_node_config_node_type import WorkflowFlowchartNodeConfigNodeType
from ..models.workflow_router_node_details import WorkflowRouterNodeDetails
from ..models.workflow_task_node_details import WorkflowTaskNodeDetails
from ..models.workflow_terminal_node_details import WorkflowTerminalNodeDetails
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowFlowchartNodeConfig")


@attr.s(auto_attribs=True, repr=False)
class WorkflowFlowchartNodeConfig:
    """  """

    _id: Union[Unset, str] = UNSET
    _node_details: Union[
        Unset, WorkflowTaskNodeDetails, WorkflowRouterNodeDetails, WorkflowTerminalNodeDetails, UnknownType
    ] = UNSET
    _node_type: Union[Unset, WorkflowFlowchartNodeConfigNodeType] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("id={}".format(repr(self._id)))
        fields.append("node_details={}".format(repr(self._node_details)))
        fields.append("node_type={}".format(repr(self._node_type)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WorkflowFlowchartNodeConfig({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        id = self._id
        node_details: Union[Unset, Dict[str, Any]]
        if isinstance(self._node_details, Unset):
            node_details = UNSET
        elif isinstance(self._node_details, UnknownType):
            node_details = self._node_details.value
        elif isinstance(self._node_details, WorkflowTaskNodeDetails):
            node_details = UNSET
            if not isinstance(self._node_details, Unset):
                node_details = self._node_details.to_dict()

        elif isinstance(self._node_details, WorkflowRouterNodeDetails):
            node_details = UNSET
            if not isinstance(self._node_details, Unset):
                node_details = self._node_details.to_dict()

        else:
            node_details = UNSET
            if not isinstance(self._node_details, Unset):
                node_details = self._node_details.to_dict()

        node_type: Union[Unset, int] = UNSET
        if not isinstance(self._node_type, Unset):
            node_type = self._node_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if id is not UNSET:
            field_dict["id"] = id
        if node_details is not UNSET:
            field_dict["nodeDetails"] = node_details
        if node_type is not UNSET:
            field_dict["nodeType"] = node_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_node_details() -> Union[
            Unset,
            WorkflowTaskNodeDetails,
            WorkflowRouterNodeDetails,
            WorkflowTerminalNodeDetails,
            UnknownType,
        ]:
            def _parse_node_details(
                data: Union[Unset, Dict[str, Any]]
            ) -> Union[
                Unset,
                WorkflowTaskNodeDetails,
                WorkflowRouterNodeDetails,
                WorkflowTerminalNodeDetails,
                UnknownType,
            ]:
                node_details: Union[
                    Unset,
                    WorkflowTaskNodeDetails,
                    WorkflowRouterNodeDetails,
                    WorkflowTerminalNodeDetails,
                    UnknownType,
                ]
                if isinstance(data, Unset):
                    return data
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    node_details = UNSET
                    _node_details = data

                    if not isinstance(_node_details, Unset):
                        node_details = WorkflowTaskNodeDetails.from_dict(_node_details)

                    return node_details
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    node_details = UNSET
                    _node_details = data

                    if not isinstance(_node_details, Unset):
                        node_details = WorkflowRouterNodeDetails.from_dict(_node_details)

                    return node_details
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    node_details = UNSET
                    _node_details = data

                    if not isinstance(_node_details, Unset):
                        node_details = WorkflowTerminalNodeDetails.from_dict(_node_details)

                    return node_details
                except:  # noqa: E722
                    pass
                return UnknownType(data)

            node_details = _parse_node_details(d.pop("nodeDetails"))

            return node_details

        try:
            node_details = get_node_details()
        except KeyError:
            if strict:
                raise
            node_details = cast(
                Union[
                    Unset,
                    WorkflowTaskNodeDetails,
                    WorkflowRouterNodeDetails,
                    WorkflowTerminalNodeDetails,
                    UnknownType,
                ],
                UNSET,
            )

        def get_node_type() -> Union[Unset, WorkflowFlowchartNodeConfigNodeType]:
            node_type = UNSET
            _node_type = d.pop("nodeType")
            if _node_type is not None and _node_type is not UNSET:
                try:
                    node_type = WorkflowFlowchartNodeConfigNodeType(_node_type)
                except ValueError:
                    node_type = WorkflowFlowchartNodeConfigNodeType.of_unknown(_node_type)

            return node_type

        try:
            node_type = get_node_type()
        except KeyError:
            if strict:
                raise
            node_type = cast(Union[Unset, WorkflowFlowchartNodeConfigNodeType], UNSET)

        workflow_flowchart_node_config = cls(
            id=id,
            node_details=node_details,
            node_type=node_type,
        )

        workflow_flowchart_node_config.additional_properties = d
        return workflow_flowchart_node_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def id(self) -> str:
        """ The ID of the workflow flowchart node config """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def node_details(
        self,
    ) -> Union[WorkflowTaskNodeDetails, WorkflowRouterNodeDetails, WorkflowTerminalNodeDetails, UnknownType]:
        if isinstance(self._node_details, Unset):
            raise NotPresentError(self, "node_details")
        return self._node_details

    @node_details.setter
    def node_details(
        self,
        value: Union[
            WorkflowTaskNodeDetails, WorkflowRouterNodeDetails, WorkflowTerminalNodeDetails, UnknownType
        ],
    ) -> None:
        self._node_details = value

    @node_details.deleter
    def node_details(self) -> None:
        self._node_details = UNSET

    @property
    def node_type(self) -> WorkflowFlowchartNodeConfigNodeType:
        """ The type associated with the node config """
        if isinstance(self._node_type, Unset):
            raise NotPresentError(self, "node_type")
        return self._node_type

    @node_type.setter
    def node_type(self, value: WorkflowFlowchartNodeConfigNodeType) -> None:
        self._node_type = value

    @node_type.deleter
    def node_type(self) -> None:
        self._node_type = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.canvas import Canvas
from ..types import UNSET, Unset

T = TypeVar("T", bound="CanvasesPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class CanvasesPaginatedList:
    """  """

    _canvases: Union[Unset, List[Canvas]] = UNSET
    _next_token: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("canvases={}".format(repr(self._canvases)))
        fields.append("next_token={}".format(repr(self._next_token)))
        return "CanvasesPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        canvases: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._canvases, Unset):
            canvases = []
            for canvases_item_data in self._canvases:
                canvases_item = canvases_item_data.to_dict()

                canvases.append(canvases_item)

        next_token = self._next_token

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if canvases is not UNSET:
            field_dict["canvases"] = canvases
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_canvases() -> Union[Unset, List[Canvas]]:
            canvases = []
            _canvases = d.pop("canvases")
            for canvases_item_data in _canvases or []:
                canvases_item = Canvas.from_dict(canvases_item_data, strict=False)

                canvases.append(canvases_item)

            return canvases

        try:
            canvases = get_canvases()
        except KeyError:
            if strict:
                raise
            canvases = cast(Union[Unset, List[Canvas]], UNSET)

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        canvases_paginated_list = cls(
            canvases=canvases,
            next_token=next_token,
        )

        return canvases_paginated_list

    @property
    def canvases(self) -> List[Canvas]:
        if isinstance(self._canvases, Unset):
            raise NotPresentError(self, "canvases")
        return self._canvases

    @canvases.setter
    def canvases(self, value: List[Canvas]) -> None:
        self._canvases = value

    @canvases.deleter
    def canvases(self) -> None:
        self._canvases = UNSET

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET



from typing import Any, cast, Dict, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.dna_sequences_match_bases_archive_reason import DnaSequencesMatchBasesArchiveReason
from ..models.dna_sequences_match_bases_sort import DnaSequencesMatchBasesSort
from ..types import UNSET, Unset

T = TypeVar("T", bound="DnaSequencesMatchBases")


@attr.s(auto_attribs=True, repr=False)
class DnaSequencesMatchBases:
    """  """

    _bases: str
    _archive_reason: Union[
        Unset, DnaSequencesMatchBasesArchiveReason
    ] = DnaSequencesMatchBasesArchiveReason.NOT_ARCHIVED
    _next_token: Union[Unset, str] = UNSET
    _page_size: Union[Unset, int] = 50
    _registry_id: Union[Unset, None, str] = UNSET
    _sort: Union[Unset, DnaSequencesMatchBasesSort] = DnaSequencesMatchBasesSort.MODIFIEDATDESC

    def __repr__(self):
        fields = []
        fields.append("bases={}".format(repr(self._bases)))
        fields.append("archive_reason={}".format(repr(self._archive_reason)))
        fields.append("next_token={}".format(repr(self._next_token)))
        fields.append("page_size={}".format(repr(self._page_size)))
        fields.append("registry_id={}".format(repr(self._registry_id)))
        fields.append("sort={}".format(repr(self._sort)))
        return "DnaSequencesMatchBases({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        bases = self._bases
        archive_reason: Union[Unset, int] = UNSET
        if not isinstance(self._archive_reason, Unset):
            archive_reason = self._archive_reason.value

        next_token = self._next_token
        page_size = self._page_size
        registry_id = self._registry_id
        sort: Union[Unset, int] = UNSET
        if not isinstance(self._sort, Unset):
            sort = self._sort.value

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if bases is not UNSET:
            field_dict["bases"] = bases
        if archive_reason is not UNSET:
            field_dict["archiveReason"] = archive_reason
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token
        if page_size is not UNSET:
            field_dict["pageSize"] = page_size
        if registry_id is not UNSET:
            field_dict["registryId"] = registry_id
        if sort is not UNSET:
            field_dict["sort"] = sort

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_bases() -> str:
            bases = d.pop("bases")
            return bases

        try:
            bases = get_bases()
        except KeyError:
            if strict:
                raise
            bases = cast(str, UNSET)

        def get_archive_reason() -> Union[Unset, DnaSequencesMatchBasesArchiveReason]:
            archive_reason = UNSET
            _archive_reason = d.pop("archiveReason")
            if _archive_reason is not None and _archive_reason is not UNSET:
                try:
                    archive_reason = DnaSequencesMatchBasesArchiveReason(_archive_reason)
                except ValueError:
                    archive_reason = DnaSequencesMatchBasesArchiveReason.of_unknown(_archive_reason)

            return archive_reason

        try:
            archive_reason = get_archive_reason()
        except KeyError:
            if strict:
                raise
            archive_reason = cast(Union[Unset, DnaSequencesMatchBasesArchiveReason], UNSET)

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        def get_page_size() -> Union[Unset, int]:
            page_size = d.pop("pageSize")
            return page_size

        try:
            page_size = get_page_size()
        except KeyError:
            if strict:
                raise
            page_size = cast(Union[Unset, int], UNSET)

        def get_registry_id() -> Union[Unset, None, str]:
            registry_id = d.pop("registryId")
            return registry_id

        try:
            registry_id = get_registry_id()
        except KeyError:
            if strict:
                raise
            registry_id = cast(Union[Unset, None, str], UNSET)

        def get_sort() -> Union[Unset, DnaSequencesMatchBasesSort]:
            sort = UNSET
            _sort = d.pop("sort")
            if _sort is not None and _sort is not UNSET:
                try:
                    sort = DnaSequencesMatchBasesSort(_sort)
                except ValueError:
                    sort = DnaSequencesMatchBasesSort.of_unknown(_sort)

            return sort

        try:
            sort = get_sort()
        except KeyError:
            if strict:
                raise
            sort = cast(Union[Unset, DnaSequencesMatchBasesSort], UNSET)

        dna_sequences_match_bases = cls(
            bases=bases,
            archive_reason=archive_reason,
            next_token=next_token,
            page_size=page_size,
            registry_id=registry_id,
            sort=sort,
        )

        return dna_sequences_match_bases

    @property
    def bases(self) -> str:
        if isinstance(self._bases, Unset):
            raise NotPresentError(self, "bases")
        return self._bases

    @bases.setter
    def bases(self, value: str) -> None:
        self._bases = value

    @property
    def archive_reason(self) -> DnaSequencesMatchBasesArchiveReason:
        if isinstance(self._archive_reason, Unset):
            raise NotPresentError(self, "archive_reason")
        return self._archive_reason

    @archive_reason.setter
    def archive_reason(self, value: DnaSequencesMatchBasesArchiveReason) -> None:
        self._archive_reason = value

    @archive_reason.deleter
    def archive_reason(self) -> None:
        self._archive_reason = UNSET

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET

    @property
    def page_size(self) -> int:
        if isinstance(self._page_size, Unset):
            raise NotPresentError(self, "page_size")
        return self._page_size

    @page_size.setter
    def page_size(self, value: int) -> None:
        self._page_size = value

    @page_size.deleter
    def page_size(self) -> None:
        self._page_size = UNSET

    @property
    def registry_id(self) -> Optional[str]:
        """ID of a registry. Restricts results to those registered in this registry. Specifying `null` returns unregistered items."""
        if isinstance(self._registry_id, Unset):
            raise NotPresentError(self, "registry_id")
        return self._registry_id

    @registry_id.setter
    def registry_id(self, value: Optional[str]) -> None:
        self._registry_id = value

    @registry_id.deleter
    def registry_id(self) -> None:
        self._registry_id = UNSET

    @property
    def sort(self) -> DnaSequencesMatchBasesSort:
        if isinstance(self._sort, Unset):
            raise NotPresentError(self, "sort")
        return self._sort

    @sort.setter
    def sort(self, value: DnaSequencesMatchBasesSort) -> None:
        self._sort = value

    @sort.deleter
    def sort(self) -> None:
        self._sort = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.barcode_scan_form_instance_provider import BarcodeScanFormInstanceProvider
from ..models.entity_search_form_instance_provider import EntitySearchFormInstanceProvider
from ..models.nested_form_definition import NestedFormDefinition
from ..models.nested_form_field_type import NestedFormFieldType
from ..models.new_blank_form_instance_provider import NewBlankFormInstanceProvider
from ..types import UNSET, Unset

T = TypeVar("T", bound="NestedFormField")


@attr.s(auto_attribs=True, repr=False)
class NestedFormField:
    """  """

    _definition: Union[Unset, NestedFormDefinition] = UNSET
    _instance_providers: Union[
        Unset,
        List[
            Union[
                BarcodeScanFormInstanceProvider,
                EntitySearchFormInstanceProvider,
                NewBlankFormInstanceProvider,
                UnknownType,
            ]
        ],
    ] = UNSET
    _type: Union[Unset, NestedFormFieldType] = UNSET
    _description: Union[Unset, str] = UNSET
    _is_required: Union[Unset, bool] = UNSET
    _key: Union[Unset, str] = UNSET
    _label: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("definition={}".format(repr(self._definition)))
        fields.append("instance_providers={}".format(repr(self._instance_providers)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("description={}".format(repr(self._description)))
        fields.append("is_required={}".format(repr(self._is_required)))
        fields.append("key={}".format(repr(self._key)))
        fields.append("label={}".format(repr(self._label)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "NestedFormField({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        definition: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._definition, Unset):
            definition = self._definition.to_dict()

        instance_providers: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._instance_providers, Unset):
            instance_providers = []
            for instance_providers_item_data in self._instance_providers:
                if isinstance(instance_providers_item_data, UnknownType):
                    instance_providers_item = instance_providers_item_data.value
                elif isinstance(instance_providers_item_data, BarcodeScanFormInstanceProvider):
                    instance_providers_item = instance_providers_item_data.to_dict()

                elif isinstance(instance_providers_item_data, EntitySearchFormInstanceProvider):
                    instance_providers_item = instance_providers_item_data.to_dict()

                else:
                    instance_providers_item = instance_providers_item_data.to_dict()

                instance_providers.append(instance_providers_item)

        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        description = self._description
        is_required = self._is_required
        key = self._key
        label = self._label

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if definition is not UNSET:
            field_dict["definition"] = definition
        if instance_providers is not UNSET:
            field_dict["instanceProviders"] = instance_providers
        if type is not UNSET:
            field_dict["type"] = type
        if description is not UNSET:
            field_dict["description"] = description
        if is_required is not UNSET:
            field_dict["isRequired"] = is_required
        if key is not UNSET:
            field_dict["key"] = key
        if label is not UNSET:
            field_dict["label"] = label

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_definition() -> Union[Unset, NestedFormDefinition]:
            definition: Union[Unset, Union[Unset, NestedFormDefinition]] = UNSET
            _definition = d.pop("definition")

            if not isinstance(_definition, Unset):
                definition = NestedFormDefinition.from_dict(_definition)

            return definition

        try:
            definition = get_definition()
        except KeyError:
            if strict:
                raise
            definition = cast(Union[Unset, NestedFormDefinition], UNSET)

        def get_instance_providers() -> Union[
            Unset,
            List[
                Union[
                    BarcodeScanFormInstanceProvider,
                    EntitySearchFormInstanceProvider,
                    NewBlankFormInstanceProvider,
                    UnknownType,
                ]
            ],
        ]:
            instance_providers = []
            _instance_providers = d.pop("instanceProviders")
            for instance_providers_item_data in _instance_providers or []:

                def _parse_instance_providers_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[
                    BarcodeScanFormInstanceProvider,
                    EntitySearchFormInstanceProvider,
                    NewBlankFormInstanceProvider,
                    UnknownType,
                ]:
                    instance_providers_item: Union[
                        BarcodeScanFormInstanceProvider,
                        EntitySearchFormInstanceProvider,
                        NewBlankFormInstanceProvider,
                        UnknownType,
                    ]
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        instance_providers_item = BarcodeScanFormInstanceProvider.from_dict(data, strict=True)

                        return instance_providers_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        instance_providers_item = EntitySearchFormInstanceProvider.from_dict(
                            data, strict=True
                        )

                        return instance_providers_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        instance_providers_item = NewBlankFormInstanceProvider.from_dict(data, strict=True)

                        return instance_providers_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                instance_providers_item = _parse_instance_providers_item(instance_providers_item_data)

                instance_providers.append(instance_providers_item)

            return instance_providers

        try:
            instance_providers = get_instance_providers()
        except KeyError:
            if strict:
                raise
            instance_providers = cast(
                Union[
                    Unset,
                    List[
                        Union[
                            BarcodeScanFormInstanceProvider,
                            EntitySearchFormInstanceProvider,
                            NewBlankFormInstanceProvider,
                            UnknownType,
                        ]
                    ],
                ],
                UNSET,
            )

        def get_type() -> Union[Unset, NestedFormFieldType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = NestedFormFieldType(_type)
                except ValueError:
                    type = NestedFormFieldType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, NestedFormFieldType], UNSET)

        def get_description() -> Union[Unset, str]:
            description = d.pop("description")
            return description

        try:
            description = get_description()
        except KeyError:
            if strict:
                raise
            description = cast(Union[Unset, str], UNSET)

        def get_is_required() -> Union[Unset, bool]:
            is_required = d.pop("isRequired")
            return is_required

        try:
            is_required = get_is_required()
        except KeyError:
            if strict:
                raise
            is_required = cast(Union[Unset, bool], UNSET)

        def get_key() -> Union[Unset, str]:
            key = d.pop("key")
            return key

        try:
            key = get_key()
        except KeyError:
            if strict:
                raise
            key = cast(Union[Unset, str], UNSET)

        def get_label() -> Union[Unset, str]:
            label = d.pop("label")
            return label

        try:
            label = get_label()
        except KeyError:
            if strict:
                raise
            label = cast(Union[Unset, str], UNSET)

        nested_form_field = cls(
            definition=definition,
            instance_providers=instance_providers,
            type=type,
            description=description,
            is_required=is_required,
            key=key,
            label=label,
        )

        nested_form_field.additional_properties = d
        return nested_form_field

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def definition(self) -> NestedFormDefinition:
        if isinstance(self._definition, Unset):
            raise NotPresentError(self, "definition")
        return self._definition

    @definition.setter
    def definition(self, value: NestedFormDefinition) -> None:
        self._definition = value

    @definition.deleter
    def definition(self) -> None:
        self._definition = UNSET

    @property
    def instance_providers(
        self,
    ) -> List[
        Union[
            BarcodeScanFormInstanceProvider,
            EntitySearchFormInstanceProvider,
            NewBlankFormInstanceProvider,
            UnknownType,
        ]
    ]:
        """Form instance declare how users can generate new instances of nested forms within a nested form field. The most obvious way to do this is just to add a new blank form; however, very often we want to describe behavior like everytime I scan a barcode, please add a new instance of this nested form with that barcode being resolved to an entity in a specific field."""
        if isinstance(self._instance_providers, Unset):
            raise NotPresentError(self, "instance_providers")
        return self._instance_providers

    @instance_providers.setter
    def instance_providers(
        self,
        value: List[
            Union[
                BarcodeScanFormInstanceProvider,
                EntitySearchFormInstanceProvider,
                NewBlankFormInstanceProvider,
                UnknownType,
            ]
        ],
    ) -> None:
        self._instance_providers = value

    @instance_providers.deleter
    def instance_providers(self) -> None:
        self._instance_providers = UNSET

    @property
    def type(self) -> NestedFormFieldType:
        """The type of this form field. Type declares how this field behaves and dictates the additional properties passed along with the required properties like label and key"""
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: NestedFormFieldType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET

    @property
    def description(self) -> str:
        """ Description of the purpose of this field """
        if isinstance(self._description, Unset):
            raise NotPresentError(self, "description")
        return self._description

    @description.setter
    def description(self, value: str) -> None:
        self._description = value

    @description.deleter
    def description(self) -> None:
        self._description = UNSET

    @property
    def is_required(self) -> bool:
        """ Whether this field is required to be filled out in order to be a valid submission """
        if isinstance(self._is_required, Unset):
            raise NotPresentError(self, "is_required")
        return self._is_required

    @is_required.setter
    def is_required(self, value: bool) -> None:
        self._is_required = value

    @is_required.deleter
    def is_required(self) -> None:
        self._is_required = UNSET

    @property
    def key(self) -> str:
        """ Reference key of this form field. Used to fix identity of fields beyond the label """
        if isinstance(self._key, Unset):
            raise NotPresentError(self, "key")
        return self._key

    @key.setter
    def key(self, value: str) -> None:
        self._key = value

    @key.deleter
    def key(self) -> None:
        self._key = UNSET

    @property
    def label(self) -> str:
        """ End user facing name of this form field. What you see when you fill out the form each time """
        if isinstance(self._label, Unset):
            raise NotPresentError(self, "label")
        return self._label

    @label.setter
    def label(self, value: str) -> None:
        self._label = value

    @label.deleter
    def label(self) -> None:
        self._label = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.dna_oligo import DnaOligo
from ..types import UNSET, Unset

T = TypeVar("T", bound="BulkCreateDnaOligosAsyncTaskResponse")


@attr.s(auto_attribs=True, repr=False)
class BulkCreateDnaOligosAsyncTaskResponse:
    """  """

    _dna_oligos: Union[Unset, List[DnaOligo]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("dna_oligos={}".format(repr(self._dna_oligos)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BulkCreateDnaOligosAsyncTaskResponse({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        dna_oligos: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._dna_oligos, Unset):
            dna_oligos = []
            for dna_oligos_item_data in self._dna_oligos:
                dna_oligos_item = dna_oligos_item_data.to_dict()

                dna_oligos.append(dna_oligos_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if dna_oligos is not UNSET:
            field_dict["dnaOligos"] = dna_oligos

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_dna_oligos() -> Union[Unset, List[DnaOligo]]:
            dna_oligos = []
            _dna_oligos = d.pop("dnaOligos")
            for dna_oligos_item_data in _dna_oligos or []:
                dna_oligos_item = DnaOligo.from_dict(dna_oligos_item_data, strict=False)

                dna_oligos.append(dna_oligos_item)

            return dna_oligos

        try:
            dna_oligos = get_dna_oligos()
        except KeyError:
            if strict:
                raise
            dna_oligos = cast(Union[Unset, List[DnaOligo]], UNSET)

        bulk_create_dna_oligos_async_task_response = cls(
            dna_oligos=dna_oligos,
        )

        bulk_create_dna_oligos_async_task_response.additional_properties = d
        return bulk_create_dna_oligos_async_task_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def dna_oligos(self) -> List[DnaOligo]:
        if isinstance(self._dna_oligos, Unset):
            raise NotPresentError(self, "dna_oligos")
        return self._dna_oligos

    @dna_oligos.setter
    def dna_oligos(self, value: List[DnaOligo]) -> None:
        self._dna_oligos = value

    @dna_oligos.deleter
    def dna_oligos(self) -> None:
        self._dna_oligos = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="RequestSampleWithBatch")


@attr.s(auto_attribs=True, repr=False)
class RequestSampleWithBatch:
    """  """

    _batch_id: str
    _container_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("batch_id={}".format(repr(self._batch_id)))
        fields.append("container_id={}".format(repr(self._container_id)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "RequestSampleWithBatch({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        batch_id = self._batch_id
        container_id = self._container_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if batch_id is not UNSET:
            field_dict["batchId"] = batch_id
        if container_id is not UNSET:
            field_dict["containerId"] = container_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_batch_id() -> str:
            batch_id = d.pop("batchId")
            return batch_id

        try:
            batch_id = get_batch_id()
        except KeyError:
            if strict:
                raise
            batch_id = cast(str, UNSET)

        def get_container_id() -> Union[Unset, str]:
            container_id = d.pop("containerId")
            return container_id

        try:
            container_id = get_container_id()
        except KeyError:
            if strict:
                raise
            container_id = cast(Union[Unset, str], UNSET)

        request_sample_with_batch = cls(
            batch_id=batch_id,
            container_id=container_id,
        )

        request_sample_with_batch.additional_properties = d
        return request_sample_with_batch

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def batch_id(self) -> str:
        if isinstance(self._batch_id, Unset):
            raise NotPresentError(self, "batch_id")
        return self._batch_id

    @batch_id.setter
    def batch_id(self, value: str) -> None:
        self._batch_id = value

    @property
    def container_id(self) -> str:
        if isinstance(self._container_id, Unset):
            raise NotPresentError(self, "container_id")
        return self._container_id

    @container_id.setter
    def container_id(self, value: str) -> None:
        self._container_id = value

    @container_id.deleter
    def container_id(self) -> None:
        self._container_id = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.ingredient_measurement_units import IngredientMeasurementUnits
from ..types import UNSET, Unset

T = TypeVar("T", bound="IngredientWriteParams")


@attr.s(auto_attribs=True, repr=False)
class IngredientWriteParams:
    """  """

    _component_entity_id: str
    _units: IngredientMeasurementUnits
    _amount: Optional[str]
    _catalog_identifier: Optional[str]
    _component_lot_container_id: Optional[str]
    _component_lot_entity_id: Optional[str]
    _component_lot_text: Optional[str]
    _notes: Optional[str]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("component_entity_id={}".format(repr(self._component_entity_id)))
        fields.append("units={}".format(repr(self._units)))
        fields.append("amount={}".format(repr(self._amount)))
        fields.append("catalog_identifier={}".format(repr(self._catalog_identifier)))
        fields.append("component_lot_container_id={}".format(repr(self._component_lot_container_id)))
        fields.append("component_lot_entity_id={}".format(repr(self._component_lot_entity_id)))
        fields.append("component_lot_text={}".format(repr(self._component_lot_text)))
        fields.append("notes={}".format(repr(self._notes)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "IngredientWriteParams({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        component_entity_id = self._component_entity_id
        units = self._units.value

        amount = self._amount
        catalog_identifier = self._catalog_identifier
        component_lot_container_id = self._component_lot_container_id
        component_lot_entity_id = self._component_lot_entity_id
        component_lot_text = self._component_lot_text
        notes = self._notes

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if component_entity_id is not UNSET:
            field_dict["componentEntityId"] = component_entity_id
        if units is not UNSET:
            field_dict["units"] = units
        if amount is not UNSET:
            field_dict["amount"] = amount
        if catalog_identifier is not UNSET:
            field_dict["catalogIdentifier"] = catalog_identifier
        if component_lot_container_id is not UNSET:
            field_dict["componentLotContainerId"] = component_lot_container_id
        if component_lot_entity_id is not UNSET:
            field_dict["componentLotEntityId"] = component_lot_entity_id
        if component_lot_text is not UNSET:
            field_dict["componentLotText"] = component_lot_text
        if notes is not UNSET:
            field_dict["notes"] = notes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_component_entity_id() -> str:
            component_entity_id = d.pop("componentEntityId")
            return component_entity_id

        try:
            component_entity_id = get_component_entity_id()
        except KeyError:
            if strict:
                raise
            component_entity_id = cast(str, UNSET)

        def get_units() -> IngredientMeasurementUnits:
            _units = d.pop("units")
            try:
                units = IngredientMeasurementUnits(_units)
            except ValueError:
                units = IngredientMeasurementUnits.of_unknown(_units)

            return units

        try:
            units = get_units()
        except KeyError:
            if strict:
                raise
            units = cast(IngredientMeasurementUnits, UNSET)

        def get_amount() -> Optional[str]:
            amount = d.pop("amount")
            return amount

        try:
            amount = get_amount()
        except KeyError:
            if strict:
                raise
            amount = cast(Optional[str], UNSET)

        def get_catalog_identifier() -> Optional[str]:
            catalog_identifier = d.pop("catalogIdentifier")
            return catalog_identifier

        try:
            catalog_identifier = get_catalog_identifier()
        except KeyError:
            if strict:
                raise
            catalog_identifier = cast(Optional[str], UNSET)

        def get_component_lot_container_id() -> Optional[str]:
            component_lot_container_id = d.pop("componentLotContainerId")
            return component_lot_container_id

        try:
            component_lot_container_id = get_component_lot_container_id()
        except KeyError:
            if strict:
                raise
            component_lot_container_id = cast(Optional[str], UNSET)

        def get_component_lot_entity_id() -> Optional[str]:
            component_lot_entity_id = d.pop("componentLotEntityId")
            return component_lot_entity_id

        try:
            component_lot_entity_id = get_component_lot_entity_id()
        except KeyError:
            if strict:
                raise
            component_lot_entity_id = cast(Optional[str], UNSET)

        def get_component_lot_text() -> Optional[str]:
            component_lot_text = d.pop("componentLotText")
            return component_lot_text

        try:
            component_lot_text = get_component_lot_text()
        except KeyError:
            if strict:
                raise
            component_lot_text = cast(Optional[str], UNSET)

        def get_notes() -> Optional[str]:
            notes = d.pop("notes")
            return notes

        try:
            notes = get_notes()
        except KeyError:
            if strict:
                raise
            notes = cast(Optional[str], UNSET)

        ingredient_write_params = cls(
            component_entity_id=component_entity_id,
            units=units,
            amount=amount,
            catalog_identifier=catalog_identifier,
            component_lot_container_id=component_lot_container_id,
            component_lot_entity_id=component_lot_entity_id,
            component_lot_text=component_lot_text,
            notes=notes,
        )

        ingredient_write_params.additional_properties = d
        return ingredient_write_params

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def component_entity_id(self) -> str:
        """ The entity that uniquely identifies this component. """
        if isinstance(self._component_entity_id, Unset):
            raise NotPresentError(self, "component_entity_id")
        return self._component_entity_id

    @component_entity_id.setter
    def component_entity_id(self, value: str) -> None:
        self._component_entity_id = value

    @property
    def units(self) -> IngredientMeasurementUnits:
        if isinstance(self._units, Unset):
            raise NotPresentError(self, "units")
        return self._units

    @units.setter
    def units(self, value: IngredientMeasurementUnits) -> None:
        self._units = value

    @property
    def amount(self) -> Optional[str]:
        """The amount value of this ingredient in its mixture, in string format (to preserve full precision). Pair with `units`. Supports scientific notation (1.23e4). One ingredient on this mixture can have an amount value of `"QS"`."""
        if isinstance(self._amount, Unset):
            raise NotPresentError(self, "amount")
        return self._amount

    @amount.setter
    def amount(self, value: Optional[str]) -> None:
        self._amount = value

    @property
    def catalog_identifier(self) -> Optional[str]:
        if isinstance(self._catalog_identifier, Unset):
            raise NotPresentError(self, "catalog_identifier")
        return self._catalog_identifier

    @catalog_identifier.setter
    def catalog_identifier(self, value: Optional[str]) -> None:
        self._catalog_identifier = value

    @property
    def component_lot_container_id(self) -> Optional[str]:
        """ The container representing the component lot for this ingredient. This is only writable if the mixture schema supports component lots in "inventory" format. """
        if isinstance(self._component_lot_container_id, Unset):
            raise NotPresentError(self, "component_lot_container_id")
        return self._component_lot_container_id

    @component_lot_container_id.setter
    def component_lot_container_id(self, value: Optional[str]) -> None:
        self._component_lot_container_id = value

    @property
    def component_lot_entity_id(self) -> Optional[str]:
        """ The entity representing the component lot for this ingredient. This is only writable if the mixture schema supports component lots in "inventory" format. """
        if isinstance(self._component_lot_entity_id, Unset):
            raise NotPresentError(self, "component_lot_entity_id")
        return self._component_lot_entity_id

    @component_lot_entity_id.setter
    def component_lot_entity_id(self, value: Optional[str]) -> None:
        self._component_lot_entity_id = value

    @property
    def component_lot_text(self) -> Optional[str]:
        """ Text representing the component lot for this ingredient. This is only writable if the mixture schema supports component lots in "text" format. """
        if isinstance(self._component_lot_text, Unset):
            raise NotPresentError(self, "component_lot_text")
        return self._component_lot_text

    @component_lot_text.setter
    def component_lot_text(self, value: Optional[str]) -> None:
        self._component_lot_text = value

    @property
    def notes(self) -> Optional[str]:
        if isinstance(self._notes, Unset):
            raise NotPresentError(self, "notes")
        return self._notes

    @notes.setter
    def notes(self, value: Optional[str]) -> None:
        self._notes = value



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.user_validation_validation_status import UserValidationValidationStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="UserValidation")


@attr.s(auto_attribs=True, repr=False)
class UserValidation:
    """  """

    _validation_comment: Union[Unset, str] = UNSET
    _validation_status: Union[Unset, UserValidationValidationStatus] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("validation_comment={}".format(repr(self._validation_comment)))
        fields.append("validation_status={}".format(repr(self._validation_status)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "UserValidation({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        validation_comment = self._validation_comment
        validation_status: Union[Unset, int] = UNSET
        if not isinstance(self._validation_status, Unset):
            validation_status = self._validation_status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if validation_comment is not UNSET:
            field_dict["validationComment"] = validation_comment
        if validation_status is not UNSET:
            field_dict["validationStatus"] = validation_status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_validation_comment() -> Union[Unset, str]:
            validation_comment = d.pop("validationComment")
            return validation_comment

        try:
            validation_comment = get_validation_comment()
        except KeyError:
            if strict:
                raise
            validation_comment = cast(Union[Unset, str], UNSET)

        def get_validation_status() -> Union[Unset, UserValidationValidationStatus]:
            validation_status = UNSET
            _validation_status = d.pop("validationStatus")
            if _validation_status is not None and _validation_status is not UNSET:
                try:
                    validation_status = UserValidationValidationStatus(_validation_status)
                except ValueError:
                    validation_status = UserValidationValidationStatus.of_unknown(_validation_status)

            return validation_status

        try:
            validation_status = get_validation_status()
        except KeyError:
            if strict:
                raise
            validation_status = cast(Union[Unset, UserValidationValidationStatus], UNSET)

        user_validation = cls(
            validation_comment=validation_comment,
            validation_status=validation_status,
        )

        user_validation.additional_properties = d
        return user_validation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def validation_comment(self) -> str:
        """ A string explaining the reason for the provided validation status. """
        if isinstance(self._validation_comment, Unset):
            raise NotPresentError(self, "validation_comment")
        return self._validation_comment

    @validation_comment.setter
    def validation_comment(self, value: str) -> None:
        self._validation_comment = value

    @validation_comment.deleter
    def validation_comment(self) -> None:
        self._validation_comment = UNSET

    @property
    def validation_status(self) -> UserValidationValidationStatus:
        """Valid values for this enum depend on whether it is used to set a value (e.g., in a POST request), or is a return value for an existing result.
        Acceptable values for setting a status are 'VALID' or 'INVALID'. Possible return values are 'VALID', 'INVALID', or 'PARTIALLY_VALID' (a result will be partially valid if it has some valid fields and some invalid fields).
        """
        if isinstance(self._validation_status, Unset):
            raise NotPresentError(self, "validation_status")
        return self._validation_status

    @validation_status.setter
    def validation_status(self, value: UserValidationValidationStatus) -> None:
        self._validation_status = value

    @validation_status.deleter
    def validation_status(self) -> None:
        self._validation_status = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.sequence_feature_custom_field import SequenceFeatureCustomField
from ..models.translation_genetic_code import TranslationGeneticCode
from ..models.translation_regions_item import TranslationRegionsItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="Translation")


@attr.s(auto_attribs=True, repr=False)
class Translation:
    """  """

    _amino_acids: Union[Unset, str] = UNSET
    _end: Union[Unset, int] = UNSET
    _genetic_code: Union[Unset, TranslationGeneticCode] = UNSET
    _regions: Union[Unset, List[TranslationRegionsItem]] = UNSET
    _start: Union[Unset, int] = UNSET
    _strand: Union[Unset, int] = UNSET
    _color: Union[Unset, str] = UNSET
    _custom_fields: Union[Unset, List[SequenceFeatureCustomField]] = UNSET
    _name: Union[Unset, str] = UNSET
    _notes: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("amino_acids={}".format(repr(self._amino_acids)))
        fields.append("end={}".format(repr(self._end)))
        fields.append("genetic_code={}".format(repr(self._genetic_code)))
        fields.append("regions={}".format(repr(self._regions)))
        fields.append("start={}".format(repr(self._start)))
        fields.append("strand={}".format(repr(self._strand)))
        fields.append("color={}".format(repr(self._color)))
        fields.append("custom_fields={}".format(repr(self._custom_fields)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("notes={}".format(repr(self._notes)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "Translation({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        amino_acids = self._amino_acids
        end = self._end
        genetic_code: Union[Unset, int] = UNSET
        if not isinstance(self._genetic_code, Unset):
            genetic_code = self._genetic_code.value

        regions: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._regions, Unset):
            regions = []
            for regions_item_data in self._regions:
                regions_item = regions_item_data.to_dict()

                regions.append(regions_item)

        start = self._start
        strand = self._strand
        color = self._color
        custom_fields: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._custom_fields, Unset):
            custom_fields = []
            for custom_fields_item_data in self._custom_fields:
                custom_fields_item = custom_fields_item_data.to_dict()

                custom_fields.append(custom_fields_item)

        name = self._name
        notes = self._notes

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if amino_acids is not UNSET:
            field_dict["aminoAcids"] = amino_acids
        if end is not UNSET:
            field_dict["end"] = end
        if genetic_code is not UNSET:
            field_dict["geneticCode"] = genetic_code
        if regions is not UNSET:
            field_dict["regions"] = regions
        if start is not UNSET:
            field_dict["start"] = start
        if strand is not UNSET:
            field_dict["strand"] = strand
        if color is not UNSET:
            field_dict["color"] = color
        if custom_fields is not UNSET:
            field_dict["customFields"] = custom_fields
        if name is not UNSET:
            field_dict["name"] = name
        if notes is not UNSET:
            field_dict["notes"] = notes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_amino_acids() -> Union[Unset, str]:
            amino_acids = d.pop("aminoAcids")
            return amino_acids

        try:
            amino_acids = get_amino_acids()
        except KeyError:
            if strict:
                raise
            amino_acids = cast(Union[Unset, str], UNSET)

        def get_end() -> Union[Unset, int]:
            end = d.pop("end")
            return end

        try:
            end = get_end()
        except KeyError:
            if strict:
                raise
            end = cast(Union[Unset, int], UNSET)

        def get_genetic_code() -> Union[Unset, TranslationGeneticCode]:
            genetic_code = UNSET
            _genetic_code = d.pop("geneticCode")
            if _genetic_code is not None and _genetic_code is not UNSET:
                try:
                    genetic_code = TranslationGeneticCode(_genetic_code)
                except ValueError:
                    genetic_code = TranslationGeneticCode.of_unknown(_genetic_code)

            return genetic_code

        try:
            genetic_code = get_genetic_code()
        except KeyError:
            if strict:
                raise
            genetic_code = cast(Union[Unset, TranslationGeneticCode], UNSET)

        def get_regions() -> Union[Unset, List[TranslationRegionsItem]]:
            regions = []
            _regions = d.pop("regions")
            for regions_item_data in _regions or []:
                regions_item = TranslationRegionsItem.from_dict(regions_item_data, strict=False)

                regions.append(regions_item)

            return regions

        try:
            regions = get_regions()
        except KeyError:
            if strict:
                raise
            regions = cast(Union[Unset, List[TranslationRegionsItem]], UNSET)

        def get_start() -> Union[Unset, int]:
            start = d.pop("start")
            return start

        try:
            start = get_start()
        except KeyError:
            if strict:
                raise
            start = cast(Union[Unset, int], UNSET)

        def get_strand() -> Union[Unset, int]:
            strand = d.pop("strand")
            return strand

        try:
            strand = get_strand()
        except KeyError:
            if strict:
                raise
            strand = cast(Union[Unset, int], UNSET)

        def get_color() -> Union[Unset, str]:
            color = d.pop("color")
            return color

        try:
            color = get_color()
        except KeyError:
            if strict:
                raise
            color = cast(Union[Unset, str], UNSET)

        def get_custom_fields() -> Union[Unset, List[SequenceFeatureCustomField]]:
            custom_fields = []
            _custom_fields = d.pop("customFields")
            for custom_fields_item_data in _custom_fields or []:
                custom_fields_item = SequenceFeatureCustomField.from_dict(
                    custom_fields_item_data, strict=False
                )

                custom_fields.append(custom_fields_item)

            return custom_fields

        try:
            custom_fields = get_custom_fields()
        except KeyError:
            if strict:
                raise
            custom_fields = cast(Union[Unset, List[SequenceFeatureCustomField]], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_notes() -> Union[Unset, str]:
            notes = d.pop("notes")
            return notes

        try:
            notes = get_notes()
        except KeyError:
            if strict:
                raise
            notes = cast(Union[Unset, str], UNSET)

        translation = cls(
            amino_acids=amino_acids,
            end=end,
            genetic_code=genetic_code,
            regions=regions,
            start=start,
            strand=strand,
            color=color,
            custom_fields=custom_fields,
            name=name,
            notes=notes,
        )

        translation.additional_properties = d
        return translation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def amino_acids(self) -> str:
        if isinstance(self._amino_acids, Unset):
            raise NotPresentError(self, "amino_acids")
        return self._amino_acids

    @amino_acids.setter
    def amino_acids(self, value: str) -> None:
        self._amino_acids = value

    @amino_acids.deleter
    def amino_acids(self) -> None:
        self._amino_acids = UNSET

    @property
    def end(self) -> int:
        if isinstance(self._end, Unset):
            raise NotPresentError(self, "end")
        return self._end

    @end.setter
    def end(self, value: int) -> None:
        self._end = value

    @end.deleter
    def end(self) -> None:
        self._end = UNSET

    @property
    def genetic_code(self) -> TranslationGeneticCode:
        """ The genetic code to use when translating the nucleotide sequence into amino acids. """
        if isinstance(self._genetic_code, Unset):
            raise NotPresentError(self, "genetic_code")
        return self._genetic_code

    @genetic_code.setter
    def genetic_code(self, value: TranslationGeneticCode) -> None:
        self._genetic_code = value

    @genetic_code.deleter
    def genetic_code(self) -> None:
        self._genetic_code = UNSET

    @property
    def regions(self) -> List[TranslationRegionsItem]:
        if isinstance(self._regions, Unset):
            raise NotPresentError(self, "regions")
        return self._regions

    @regions.setter
    def regions(self, value: List[TranslationRegionsItem]) -> None:
        self._regions = value

    @regions.deleter
    def regions(self) -> None:
        self._regions = UNSET

    @property
    def start(self) -> int:
        if isinstance(self._start, Unset):
            raise NotPresentError(self, "start")
        return self._start

    @start.setter
    def start(self, value: int) -> None:
        self._start = value

    @start.deleter
    def start(self) -> None:
        self._start = UNSET

    @property
    def strand(self) -> int:
        if isinstance(self._strand, Unset):
            raise NotPresentError(self, "strand")
        return self._strand

    @strand.setter
    def strand(self, value: int) -> None:
        self._strand = value

    @strand.deleter
    def strand(self) -> None:
        self._strand = UNSET

    @property
    def color(self) -> str:
        """ Hex color code used when displaying this feature in the UI. """
        if isinstance(self._color, Unset):
            raise NotPresentError(self, "color")
        return self._color

    @color.setter
    def color(self, value: str) -> None:
        self._color = value

    @color.deleter
    def color(self) -> None:
        self._color = UNSET

    @property
    def custom_fields(self) -> List[SequenceFeatureCustomField]:
        if isinstance(self._custom_fields, Unset):
            raise NotPresentError(self, "custom_fields")
        return self._custom_fields

    @custom_fields.setter
    def custom_fields(self, value: List[SequenceFeatureCustomField]) -> None:
        self._custom_fields = value

    @custom_fields.deleter
    def custom_fields(self) -> None:
        self._custom_fields = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def notes(self) -> str:
        if isinstance(self._notes, Unset):
            raise NotPresentError(self, "notes")
        return self._notes

    @notes.setter
    def notes(self, value: str) -> None:
        self._notes = value

    @notes.deleter
    def notes(self) -> None:
        self._notes = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.assay_fields_create import AssayFieldsCreate
from ..models.assay_run_validation_status import AssayRunValidationStatus
from ..models.fields import Fields
from ..types import UNSET, Unset

T = TypeVar("T", bound="AssayRunCreate")


@attr.s(auto_attribs=True, repr=False)
class AssayRunCreate:
    """  """

    _fields: Union[Fields, AssayFieldsCreate, UnknownType]
    _schema_id: str
    _id: Union[Unset, str] = UNSET
    _project_id: Union[Unset, str] = UNSET
    _validation_comment: Union[Unset, str] = UNSET
    _validation_status: Union[Unset, AssayRunValidationStatus] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("schema_id={}".format(repr(self._schema_id)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("project_id={}".format(repr(self._project_id)))
        fields.append("validation_comment={}".format(repr(self._validation_comment)))
        fields.append("validation_status={}".format(repr(self._validation_status)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AssayRunCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        if isinstance(self._fields, UnknownType):
            fields = self._fields.value
        elif isinstance(self._fields, Fields):
            fields = self._fields.to_dict()

        else:
            fields = self._fields.to_dict()

        schema_id = self._schema_id
        id = self._id
        project_id = self._project_id
        validation_comment = self._validation_comment
        validation_status: Union[Unset, int] = UNSET
        if not isinstance(self._validation_status, Unset):
            validation_status = self._validation_status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if fields is not UNSET:
            field_dict["fields"] = fields
        if schema_id is not UNSET:
            field_dict["schemaId"] = schema_id
        if id is not UNSET:
            field_dict["id"] = id
        if project_id is not UNSET:
            field_dict["projectId"] = project_id
        if validation_comment is not UNSET:
            field_dict["validationComment"] = validation_comment
        if validation_status is not UNSET:
            field_dict["validationStatus"] = validation_status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_fields() -> Union[Fields, AssayFieldsCreate, UnknownType]:
            def _parse_fields(data: Union[Dict[str, Any]]) -> Union[Fields, AssayFieldsCreate, UnknownType]:
                fields: Union[Fields, AssayFieldsCreate, UnknownType]
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    fields = Fields.from_dict(data, strict=True)

                    return fields
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    fields = AssayFieldsCreate.from_dict(data, strict=True)

                    return fields
                except:  # noqa: E722
                    pass
                return UnknownType(data)

            fields = _parse_fields(d.pop("fields"))

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Fields, AssayFieldsCreate, UnknownType], UNSET)

        def get_schema_id() -> str:
            schema_id = d.pop("schemaId")
            return schema_id

        try:
            schema_id = get_schema_id()
        except KeyError:
            if strict:
                raise
            schema_id = cast(str, UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_project_id() -> Union[Unset, str]:
            project_id = d.pop("projectId")
            return project_id

        try:
            project_id = get_project_id()
        except KeyError:
            if strict:
                raise
            project_id = cast(Union[Unset, str], UNSET)

        def get_validation_comment() -> Union[Unset, str]:
            validation_comment = d.pop("validationComment")
            return validation_comment

        try:
            validation_comment = get_validation_comment()
        except KeyError:
            if strict:
                raise
            validation_comment = cast(Union[Unset, str], UNSET)

        def get_validation_status() -> Union[Unset, AssayRunValidationStatus]:
            validation_status = UNSET
            _validation_status = d.pop("validationStatus")
            if _validation_status is not None and _validation_status is not UNSET:
                try:
                    validation_status = AssayRunValidationStatus(_validation_status)
                except ValueError:
                    validation_status = AssayRunValidationStatus.of_unknown(_validation_status)

            return validation_status

        try:
            validation_status = get_validation_status()
        except KeyError:
            if strict:
                raise
            validation_status = cast(Union[Unset, AssayRunValidationStatus], UNSET)

        assay_run_create = cls(
            fields=fields,
            schema_id=schema_id,
            id=id,
            project_id=project_id,
            validation_comment=validation_comment,
            validation_status=validation_status,
        )

        assay_run_create.additional_properties = d
        return assay_run_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def fields(self) -> Union[Fields, AssayFieldsCreate, UnknownType]:
        """ Object of assay run fields """
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Union[Fields, AssayFieldsCreate, UnknownType]) -> None:
        self._fields = value

    @property
    def schema_id(self) -> str:
        """ ID of assay schema that assay run conforms to """
        if isinstance(self._schema_id, Unset):
            raise NotPresentError(self, "schema_id")
        return self._schema_id

    @schema_id.setter
    def schema_id(self, value: str) -> None:
        self._schema_id = value

    @property
    def id(self) -> str:
        """ ID of assay run """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def project_id(self) -> str:
        """The project that the assay run should be uploaded to. Only users with read access to the project will be able to read the assay run. Leaving this empty will result in only the creator having read access."""
        if isinstance(self._project_id, Unset):
            raise NotPresentError(self, "project_id")
        return self._project_id

    @project_id.setter
    def project_id(self, value: str) -> None:
        self._project_id = value

    @project_id.deleter
    def project_id(self) -> None:
        self._project_id = UNSET

    @property
    def validation_comment(self) -> str:
        """ Additional information about the validation status """
        if isinstance(self._validation_comment, Unset):
            raise NotPresentError(self, "validation_comment")
        return self._validation_comment

    @validation_comment.setter
    def validation_comment(self, value: str) -> None:
        self._validation_comment = value

    @validation_comment.deleter
    def validation_comment(self) -> None:
        self._validation_comment = UNSET

    @property
    def validation_status(self) -> AssayRunValidationStatus:
        """ Must be either VALID or INVALID """
        if isinstance(self._validation_status, Unset):
            raise NotPresentError(self, "validation_status")
        return self._validation_status

    @validation_status.setter
    def validation_status(self, value: AssayRunValidationStatus) -> None:
        self._validation_status = value

    @validation_status.deleter
    def validation_status(self) -> None:
        self._validation_status = UNSET



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class ContainerQuantityUnits(Enums.KnownString):
    L = "L"
    ML = "mL"
    UL = "uL"
    NL = "nL"
    PL = "pL"
    GAL_US = "gal (US)"
    QT_US = "qt (US)"
    PT_US = "pt (US)"
    KG = "kg"
    G = "g"
    MG = "mg"
    UG = "ug"
    NG = "ng"
    PG = "pg"
    LB = "lb"
    OZ = "oz"
    MOL = "mol"
    MMOL = "mmol"
    UMOL = "umol"
    NMOL = "nmol"
    PMOL = "pmol"
    CELLS = "cells"
    VALUE_22 = "(x10^3) cells"
    VALUE_23 = "(x10^4) cells"
    VALUE_24 = "(x10^5) cells"
    VALUE_25 = "(x10^6) cells"
    VALUE_26 = "(x10^7) cells"
    VALUE_27 = "(x10^8) cells"
    VALUE_28 = "(x10^9) cells"
    ITEMS = "items"
    UNITS = "units"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "ContainerQuantityUnits":
        if not isinstance(val, str):
            raise ValueError(f"Value of ContainerQuantityUnits must be a string (encountered: {val})")
        newcls = Enum("ContainerQuantityUnits", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(ContainerQuantityUnits, getattr(newcls, "_UNKNOWN"))



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.find_matching_regions_async_task_response_aa_sequence_matches_item import (
    FindMatchingRegionsAsyncTaskResponseAaSequenceMatchesItem,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="FindMatchingRegionsAsyncTaskResponse")


@attr.s(auto_attribs=True, repr=False)
class FindMatchingRegionsAsyncTaskResponse:
    """  """

    _aa_sequence_matches: Union[
        Unset, List[FindMatchingRegionsAsyncTaskResponseAaSequenceMatchesItem]
    ] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("aa_sequence_matches={}".format(repr(self._aa_sequence_matches)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "FindMatchingRegionsAsyncTaskResponse({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        aa_sequence_matches: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._aa_sequence_matches, Unset):
            aa_sequence_matches = []
            for aa_sequence_matches_item_data in self._aa_sequence_matches:
                aa_sequence_matches_item = aa_sequence_matches_item_data.to_dict()

                aa_sequence_matches.append(aa_sequence_matches_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if aa_sequence_matches is not UNSET:
            field_dict["aaSequenceMatches"] = aa_sequence_matches

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_aa_sequence_matches() -> Union[
            Unset, List[FindMatchingRegionsAsyncTaskResponseAaSequenceMatchesItem]
        ]:
            aa_sequence_matches = []
            _aa_sequence_matches = d.pop("aaSequenceMatches")
            for aa_sequence_matches_item_data in _aa_sequence_matches or []:
                aa_sequence_matches_item = (
                    FindMatchingRegionsAsyncTaskResponseAaSequenceMatchesItem.from_dict(
                        aa_sequence_matches_item_data, strict=False
                    )
                )

                aa_sequence_matches.append(aa_sequence_matches_item)

            return aa_sequence_matches

        try:
            aa_sequence_matches = get_aa_sequence_matches()
        except KeyError:
            if strict:
                raise
            aa_sequence_matches = cast(
                Union[Unset, List[FindMatchingRegionsAsyncTaskResponseAaSequenceMatchesItem]], UNSET
            )

        find_matching_regions_async_task_response = cls(
            aa_sequence_matches=aa_sequence_matches,
        )

        find_matching_regions_async_task_response.additional_properties = d
        return find_matching_regions_async_task_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def aa_sequence_matches(self) -> List[FindMatchingRegionsAsyncTaskResponseAaSequenceMatchesItem]:
        if isinstance(self._aa_sequence_matches, Unset):
            raise NotPresentError(self, "aa_sequence_matches")
        return self._aa_sequence_matches

    @aa_sequence_matches.setter
    def aa_sequence_matches(
        self, value: List[FindMatchingRegionsAsyncTaskResponseAaSequenceMatchesItem]
    ) -> None:
        self._aa_sequence_matches = value

    @aa_sequence_matches.deleter
    def aa_sequence_matches(self) -> None:
        self._aa_sequence_matches = UNSET



import warnings

import benchling_api_client.v2.extensions

NotPresentError = benchling_api_client.v2.extensions.NotPresentError
UnknownType = benchling_api_client.v2.extensions.UnknownType
Enums = benchling_api_client.v2.extensions.Enums

warnings.warn("This package is deprecated: please use benchling_api_client.v2.extensions", DeprecationWarning)



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.entity_worklist_items_list_type import EntityWorklistItemsListType
from ..models.generic_entity import GenericEntity
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntityWorklistItemsList")


@attr.s(auto_attribs=True, repr=False)
class EntityWorklistItemsList:
    """  """

    _next_token: Union[Unset, str] = UNSET
    _type: Union[Unset, EntityWorklistItemsListType] = UNSET
    _worklist_items: Union[Unset, List[GenericEntity]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("next_token={}".format(repr(self._next_token)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("worklist_items={}".format(repr(self._worklist_items)))
        return "EntityWorklistItemsList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        next_token = self._next_token
        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        worklist_items: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._worklist_items, Unset):
            worklist_items = []
            for worklist_items_item_data in self._worklist_items:
                worklist_items_item = worklist_items_item_data.to_dict()

                worklist_items.append(worklist_items_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token
        if type is not UNSET:
            field_dict["type"] = type
        if worklist_items is not UNSET:
            field_dict["worklistItems"] = worklist_items

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        def get_type() -> Union[Unset, EntityWorklistItemsListType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = EntityWorklistItemsListType(_type)
                except ValueError:
                    type = EntityWorklistItemsListType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, EntityWorklistItemsListType], UNSET)

        def get_worklist_items() -> Union[Unset, List[GenericEntity]]:
            worklist_items = []
            _worklist_items = d.pop("worklistItems")
            for worklist_items_item_data in _worklist_items or []:
                worklist_items_item = GenericEntity.from_dict(worklist_items_item_data, strict=False)

                worklist_items.append(worklist_items_item)

            return worklist_items

        try:
            worklist_items = get_worklist_items()
        except KeyError:
            if strict:
                raise
            worklist_items = cast(Union[Unset, List[GenericEntity]], UNSET)

        entity_worklist_items_list = cls(
            next_token=next_token,
            type=type,
            worklist_items=worklist_items,
        )

        return entity_worklist_items_list

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET

    @property
    def type(self) -> EntityWorklistItemsListType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: EntityWorklistItemsListType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET

    @property
    def worklist_items(self) -> List[GenericEntity]:
        if isinstance(self._worklist_items, Unset):
            raise NotPresentError(self, "worklist_items")
        return self._worklist_items

    @worklist_items.setter
    def worklist_items(self, value: List[GenericEntity]) -> None:
        self._worklist_items = value

    @worklist_items.deleter
    def worklist_items(self) -> None:
        self._worklist_items = UNSET



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="ReducedPattern")


@attr.s(auto_attribs=True, repr=False)
class ReducedPattern:
    """Patterns must consist of ATGC bases only, and are case-insensitive ("gat" will reduce usage of "GAT"). If avoidReverseComplement is true, then the pattern's reverse complement will also be avoided."""

    _avoid_reverse_complement: Union[Unset, bool] = False
    _pattern: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("avoid_reverse_complement={}".format(repr(self._avoid_reverse_complement)))
        fields.append("pattern={}".format(repr(self._pattern)))
        return "ReducedPattern({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        avoid_reverse_complement = self._avoid_reverse_complement
        pattern = self._pattern

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if avoid_reverse_complement is not UNSET:
            field_dict["avoidReverseComplement"] = avoid_reverse_complement
        if pattern is not UNSET:
            field_dict["pattern"] = pattern

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_avoid_reverse_complement() -> Union[Unset, bool]:
            avoid_reverse_complement = d.pop("avoidReverseComplement")
            return avoid_reverse_complement

        try:
            avoid_reverse_complement = get_avoid_reverse_complement()
        except KeyError:
            if strict:
                raise
            avoid_reverse_complement = cast(Union[Unset, bool], UNSET)

        def get_pattern() -> Union[Unset, str]:
            pattern = d.pop("pattern")
            return pattern

        try:
            pattern = get_pattern()
        except KeyError:
            if strict:
                raise
            pattern = cast(Union[Unset, str], UNSET)

        reduced_pattern = cls(
            avoid_reverse_complement=avoid_reverse_complement,
            pattern=pattern,
        )

        return reduced_pattern

    @property
    def avoid_reverse_complement(self) -> bool:
        if isinstance(self._avoid_reverse_complement, Unset):
            raise NotPresentError(self, "avoid_reverse_complement")
        return self._avoid_reverse_complement

    @avoid_reverse_complement.setter
    def avoid_reverse_complement(self, value: bool) -> None:
        self._avoid_reverse_complement = value

    @avoid_reverse_complement.deleter
    def avoid_reverse_complement(self) -> None:
        self._avoid_reverse_complement = UNSET

    @property
    def pattern(self) -> str:
        if isinstance(self._pattern, Unset):
            raise NotPresentError(self, "pattern")
        return self._pattern

    @pattern.setter
    def pattern(self, value: str) -> None:
        self._pattern = value

    @pattern.deleter
    def pattern(self) -> None:
        self._pattern = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.nucleotide_alignment_base_algorithm import NucleotideAlignmentBaseAlgorithm
from ..models.nucleotide_alignment_base_files_item import NucleotideAlignmentBaseFilesItem
from ..models.nucleotide_alignment_file import NucleotideAlignmentFile
from ..models.nucleotide_consensus_alignment_create_new_sequence import (
    NucleotideConsensusAlignmentCreateNewSequence,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="NucleotideConsensusAlignmentCreate")


@attr.s(auto_attribs=True, repr=False)
class NucleotideConsensusAlignmentCreate:
    """  """

    _algorithm: NucleotideAlignmentBaseAlgorithm
    _files: List[Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]]
    _new_sequence: Union[Unset, NucleotideConsensusAlignmentCreateNewSequence] = UNSET
    _sequence_id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("algorithm={}".format(repr(self._algorithm)))
        fields.append("files={}".format(repr(self._files)))
        fields.append("new_sequence={}".format(repr(self._new_sequence)))
        fields.append("sequence_id={}".format(repr(self._sequence_id)))
        fields.append("name={}".format(repr(self._name)))
        return "NucleotideConsensusAlignmentCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        algorithm = self._algorithm.value

        files = []
        for files_item_data in self._files:
            if isinstance(files_item_data, UnknownType):
                files_item = files_item_data.value
            elif isinstance(files_item_data, NucleotideAlignmentBaseFilesItem):
                files_item = files_item_data.to_dict()

            else:
                files_item = files_item_data.to_dict()

            files.append(files_item)

        new_sequence: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._new_sequence, Unset):
            new_sequence = self._new_sequence.to_dict()

        sequence_id = self._sequence_id
        name = self._name

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if algorithm is not UNSET:
            field_dict["algorithm"] = algorithm
        if files is not UNSET:
            field_dict["files"] = files
        if new_sequence is not UNSET:
            field_dict["newSequence"] = new_sequence
        if sequence_id is not UNSET:
            field_dict["sequenceId"] = sequence_id
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_algorithm() -> NucleotideAlignmentBaseAlgorithm:
            _algorithm = d.pop("algorithm")
            try:
                algorithm = NucleotideAlignmentBaseAlgorithm(_algorithm)
            except ValueError:
                algorithm = NucleotideAlignmentBaseAlgorithm.of_unknown(_algorithm)

            return algorithm

        try:
            algorithm = get_algorithm()
        except KeyError:
            if strict:
                raise
            algorithm = cast(NucleotideAlignmentBaseAlgorithm, UNSET)

        def get_files() -> List[
            Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]
        ]:
            files = []
            _files = d.pop("files")
            for files_item_data in _files:

                def _parse_files_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]:
                    files_item: Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        files_item = NucleotideAlignmentBaseFilesItem.from_dict(data, strict=True)

                        return files_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        files_item = NucleotideAlignmentFile.from_dict(data, strict=True)

                        return files_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                files_item = _parse_files_item(files_item_data)

                files.append(files_item)

            return files

        try:
            files = get_files()
        except KeyError:
            if strict:
                raise
            files = cast(
                List[Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]], UNSET
            )

        def get_new_sequence() -> Union[Unset, NucleotideConsensusAlignmentCreateNewSequence]:
            new_sequence: Union[Unset, Union[Unset, NucleotideConsensusAlignmentCreateNewSequence]] = UNSET
            _new_sequence = d.pop("newSequence")

            if not isinstance(_new_sequence, Unset):
                new_sequence = NucleotideConsensusAlignmentCreateNewSequence.from_dict(_new_sequence)

            return new_sequence

        try:
            new_sequence = get_new_sequence()
        except KeyError:
            if strict:
                raise
            new_sequence = cast(Union[Unset, NucleotideConsensusAlignmentCreateNewSequence], UNSET)

        def get_sequence_id() -> Union[Unset, str]:
            sequence_id = d.pop("sequenceId")
            return sequence_id

        try:
            sequence_id = get_sequence_id()
        except KeyError:
            if strict:
                raise
            sequence_id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        nucleotide_consensus_alignment_create = cls(
            algorithm=algorithm,
            files=files,
            new_sequence=new_sequence,
            sequence_id=sequence_id,
            name=name,
        )

        return nucleotide_consensus_alignment_create

    @property
    def algorithm(self) -> NucleotideAlignmentBaseAlgorithm:
        if isinstance(self._algorithm, Unset):
            raise NotPresentError(self, "algorithm")
        return self._algorithm

    @algorithm.setter
    def algorithm(self, value: NucleotideAlignmentBaseAlgorithm) -> None:
        self._algorithm = value

    @property
    def files(self) -> List[Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]]:
        if isinstance(self._files, Unset):
            raise NotPresentError(self, "files")
        return self._files

    @files.setter
    def files(
        self, value: List[Union[NucleotideAlignmentBaseFilesItem, NucleotideAlignmentFile, UnknownType]]
    ) -> None:
        self._files = value

    @property
    def new_sequence(self) -> NucleotideConsensusAlignmentCreateNewSequence:
        if isinstance(self._new_sequence, Unset):
            raise NotPresentError(self, "new_sequence")
        return self._new_sequence

    @new_sequence.setter
    def new_sequence(self, value: NucleotideConsensusAlignmentCreateNewSequence) -> None:
        self._new_sequence = value

    @new_sequence.deleter
    def new_sequence(self) -> None:
        self._new_sequence = UNSET

    @property
    def sequence_id(self) -> str:
        if isinstance(self._sequence_id, Unset):
            raise NotPresentError(self, "sequence_id")
        return self._sequence_id

    @sequence_id.setter
    def sequence_id(self, value: str) -> None:
        self._sequence_id = value

    @sequence_id.deleter
    def sequence_id(self) -> None:
        self._sequence_id = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class SchemaDependencySubtypes(Enums.KnownString):
    AA_SEQUENCE = "aa_sequence"
    DNA_SEQUENCE = "dna_sequence"
    CUSTOM_ENTITY = "custom_entity"
    MIXTURE = "mixture"
    MOLECULE = "molecule"
    DNA_OLIGO = "dna_oligo"
    RNA_OLIGO = "rna_oligo"
    RNA_SEQUENCE = "rna_sequence"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "SchemaDependencySubtypes":
        if not isinstance(val, str):
            raise ValueError(f"Value of SchemaDependencySubtypes must be a string (encountered: {val})")
        newcls = Enum("SchemaDependencySubtypes", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(SchemaDependencySubtypes, getattr(newcls, "_UNKNOWN"))



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.archive_record import ArchiveRecord
from ..models.dropdown_option import DropdownOption
from ..types import UNSET, Unset

T = TypeVar("T", bound="Dropdown")


@attr.s(auto_attribs=True, repr=False)
class Dropdown:
    """ Dropdowns are registry-wide enums. Use dropdowns to standardize on spelling and naming conventions, especially for important metadata like resistance markers. """

    _archive_record: Union[Unset, None, ArchiveRecord] = UNSET
    _options: Union[Unset, List[DropdownOption]] = UNSET
    _id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("archive_record={}".format(repr(self._archive_record)))
        fields.append("options={}".format(repr(self._options)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "Dropdown({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        archive_record: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._archive_record, Unset):
            archive_record = self._archive_record.to_dict() if self._archive_record else None

        options: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._options, Unset):
            options = []
            for options_item_data in self._options:
                options_item = options_item_data.to_dict()

                options.append(options_item)

        id = self._id
        name = self._name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if archive_record is not UNSET:
            field_dict["archiveRecord"] = archive_record
        if options is not UNSET:
            field_dict["options"] = options
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_archive_record() -> Union[Unset, None, ArchiveRecord]:
            archive_record = None
            _archive_record = d.pop("archiveRecord")

            if _archive_record is not None and not isinstance(_archive_record, Unset):
                archive_record = ArchiveRecord.from_dict(_archive_record)

            return archive_record

        try:
            archive_record = get_archive_record()
        except KeyError:
            if strict:
                raise
            archive_record = cast(Union[Unset, None, ArchiveRecord], UNSET)

        def get_options() -> Union[Unset, List[DropdownOption]]:
            options = []
            _options = d.pop("options")
            for options_item_data in _options or []:
                options_item = DropdownOption.from_dict(options_item_data, strict=False)

                options.append(options_item)

            return options

        try:
            options = get_options()
        except KeyError:
            if strict:
                raise
            options = cast(Union[Unset, List[DropdownOption]], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        dropdown = cls(
            archive_record=archive_record,
            options=options,
            id=id,
            name=name,
        )

        dropdown.additional_properties = d
        return dropdown

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def archive_record(self) -> Optional[ArchiveRecord]:
        if isinstance(self._archive_record, Unset):
            raise NotPresentError(self, "archive_record")
        return self._archive_record

    @archive_record.setter
    def archive_record(self, value: Optional[ArchiveRecord]) -> None:
        self._archive_record = value

    @archive_record.deleter
    def archive_record(self) -> None:
        self._archive_record = UNSET

    @property
    def options(self) -> List[DropdownOption]:
        """ Array of dropdown options """
        if isinstance(self._options, Unset):
            raise NotPresentError(self, "options")
        return self._options

    @options.setter
    def options(self, value: List[DropdownOption]) -> None:
        self._options = value

    @options.deleter
    def options(self) -> None:
        self._options = UNSET

    @property
    def id(self) -> str:
        """ ID of the dropdown """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def name(self) -> str:
        """ Name of the dropdown """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET



from typing import Any, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import UnknownType
from ..models.inaccessible_resource import InaccessibleResource
from ..models.well import Well

T = TypeVar("T", bound="PlateWells")


@attr.s(auto_attribs=True, repr=False)
class PlateWells:
    """ Well contents of the plate, keyed by position string (eg. "A1"). """

    additional_properties: Dict[str, Union[Well, InaccessibleResource, UnknownType]] = attr.ib(
        init=False, factory=dict
    )

    def __repr__(self):
        fields = []
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "PlateWells({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            if isinstance(prop, UnknownType):
                field_dict[prop_name] = prop.value
            elif isinstance(prop, Well):
                field_dict[prop_name] = prop.to_dict()

            else:
                field_dict[prop_name] = prop.to_dict()

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        plate_wells = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():

            def _parse_additional_property(
                data: Union[Dict[str, Any]]
            ) -> Union[Well, InaccessibleResource, UnknownType]:
                additional_property: Union[Well, InaccessibleResource, UnknownType]
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    additional_property = Well.from_dict(data, strict=True)

                    return additional_property
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    additional_property = InaccessibleResource.from_dict(data, strict=True)

                    return additional_property
                except:  # noqa: E722
                    pass
                return UnknownType(data)

            additional_property = _parse_additional_property(prop_dict)

            additional_properties[prop_name] = additional_property

        plate_wells.additional_properties = additional_properties
        return plate_wells

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Union[Well, InaccessibleResource, UnknownType]:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Union[Well, InaccessibleResource, UnknownType]) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Union[Well, InaccessibleResource, UnknownType]]:
        return self.additional_properties.get(key, default)



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.deprecated_container_volume_for_input_units import DeprecatedContainerVolumeForInputUnits
from ..types import UNSET, Unset

T = TypeVar("T", bound="DeprecatedContainerVolumeForInput")


@attr.s(auto_attribs=True, repr=False)
class DeprecatedContainerVolumeForInput:
    """Desired volume for a container, well, or transfer. "volume" type keys are deprecated in API requests; use the more permissive "quantity" type key instead."""

    _units: Union[Unset, None, DeprecatedContainerVolumeForInputUnits] = UNSET
    _value: Union[Unset, None, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("units={}".format(repr(self._units)))
        fields.append("value={}".format(repr(self._value)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "DeprecatedContainerVolumeForInput({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        units: Union[Unset, None, int] = UNSET
        if not isinstance(self._units, Unset):
            units = self._units.value if self._units else None

        value = self._value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if units is not UNSET:
            field_dict["units"] = units
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_units() -> Union[Unset, None, DeprecatedContainerVolumeForInputUnits]:
            units = UNSET
            _units = d.pop("units")
            if _units is not None and _units is not UNSET:
                try:
                    units = DeprecatedContainerVolumeForInputUnits(_units)
                except ValueError:
                    units = DeprecatedContainerVolumeForInputUnits.of_unknown(_units)

            return units

        try:
            units = get_units()
        except KeyError:
            if strict:
                raise
            units = cast(Union[Unset, None, DeprecatedContainerVolumeForInputUnits], UNSET)

        def get_value() -> Union[Unset, None, float]:
            value = d.pop("value")
            return value

        try:
            value = get_value()
        except KeyError:
            if strict:
                raise
            value = cast(Union[Unset, None, float], UNSET)

        deprecated_container_volume_for_input = cls(
            units=units,
            value=value,
        )

        deprecated_container_volume_for_input.additional_properties = d
        return deprecated_container_volume_for_input

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def units(self) -> Optional[DeprecatedContainerVolumeForInputUnits]:
        if isinstance(self._units, Unset):
            raise NotPresentError(self, "units")
        return self._units

    @units.setter
    def units(self, value: Optional[DeprecatedContainerVolumeForInputUnits]) -> None:
        self._units = value

    @units.deleter
    def units(self) -> None:
        self._units = UNSET

    @property
    def value(self) -> Optional[float]:
        if isinstance(self._value, Unset):
            raise NotPresentError(self, "value")
        return self._value

    @value.setter
    def value(self, value: Optional[float]) -> None:
        self._value = value

    @value.deleter
    def value(self) -> None:
        self._value = UNSET
